package org.amse.mARICa.ioXML;

import java.io.*;
import java.util.ArrayList;

import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

import org.amse.mARICa.game.Point;
import org.amse.mARICa.model.*;
import org.amse.mARICa.model.impl.Board;
public class  Reader {
	
	Document myDoc;
	boolean myError;
	
	public Reader(File file){
		myDoc = parseXmlFile(file, false);				
	}
	
	public Reader(InputStream is){
		myDoc = parseXmlFile(is, false);
	}
	
	public boolean isError(){
		return myError;
	}

    // Parses an XML file and returns a DOM document.
    // If validating is true, the contents is validated against the DTD
    // specified in the file.
    /**
     * Parses XML   DOM document.
     *   Exception   scores.dat 
     *     .
     */
	private Document parseXmlFile(InputStream is, boolean validating) {
		try {
            // Create a builder factory
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(validating);

            // Create the builder and parse the file
             Document doc = factory.newDocumentBuilder().parse(is);
            return doc;
        } catch (SAXException e) { 
            // A parsing error occurred; the xml input is not valid
        		throw new ReaderException(" XML  ");
        } catch (ParserConfigurationException e) {
        		throw new ReaderException("  ");
        } catch (IOException e) {
        		throw new ReaderException("  ");
        }catch (IllegalArgumentException e){
        	//   - InputStream == null
        	myError = true;
        	return null;
        }		
    }
    /**
     * Parses XML   DOM document.
     *   Exception   scores.dat 
     *     .
     */
	private Document parseXmlFile(File file, boolean validating) {
		try {
            // Create a builder factory
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(validating);

            // Create the builder and parse the file
             Document doc = factory.newDocumentBuilder().parse(file);
            return doc;
        } catch (SAXException e) { 
            // A parsing error occurred; the xml input is not valid
        		throw new ReaderException(" XML  ");
        } catch (ParserConfigurationException e) {
        		throw new ReaderException("  ");
        } catch (IOException e) {
        		throw new ReaderException("  ");
        }catch (IllegalArgumentException e){
        	//   - InputStream == null
        	myError = true;
        	return null;
        }		
    }
    /**
     *   Board   Document,    xml
     * @param board - 
     * @return ,        
     */
    public DataForGame setBoardFromDoc(){
    	try{    		
    		IBoard board;
    		int width, height, countStore, minStep, x, y;
    		width = new Integer(myDoc.getDocumentElement().getAttribute("width")).intValue();
    		height = new Integer(myDoc.getDocumentElement().getAttribute("height")).intValue();
    		minStep = new Integer (((Element)myDoc.getElementsByTagName("minStep").item(0)).getAttribute("count")).intValue();
    		countStore = new Integer (((Element)myDoc.getElementsByTagName("store").item(0)).getAttribute("count")).intValue();
    		board = new Board(height, width);
    		
    		Point StartPos = null;
    		Point Stores [] = new Point [countStore];
    		
    		//  WALL    BOARD'
    		NodeList list = myDoc.getElementsByTagName("wall");
    		for (int i=0; i<list.getLength(); i++) {
    			// Get element
    			Element elWall = (Element)list.item(i);
    			x = new Integer(elWall.getAttribute("x")).intValue();
    			y = new Integer(elWall.getAttribute("y")).intValue();
    			if (board.getCell(x, y) != null)
    				throw new ReaderException (" XML  .   Cell   ");
    			board.setWall(x, y);           
    		}
    	
    		//  FREECELL    BOARD'
    		list = myDoc.getElementsByTagName("cell");
    		for (int i=0, k=0; i<list.getLength(); i++) {
    			// Get element
    			Element elCell = (Element)list.item(i);
    			x = new Integer(elCell.getAttribute("x")).intValue();
    			y = new Integer(elCell.getAttribute("y")).intValue();
    			boolean store = new Boolean(elCell.getAttribute("store")).booleanValue();
    			boolean man = new Boolean(elCell.getAttribute("man")).booleanValue();
    			boolean box = new Boolean(elCell.getAttribute("box")).booleanValue();
    			if (store){
    				Stores[k++] = new Point(x, y);
    			}
    			if(man){
    				StartPos = new Point(x, y);
    			}
    			if (board.getCell(x, y) != null)
    				throw new ReaderException (" XML  .   Cell   ");
    			IFreeCell cell = board.setFreeCell(x, y);
    			cell.setStore(store);
    			if (man) 
    				cell.setMan();
    			if (box) 
    				cell.setBox();
    		}    	
    		DataForGame data = new DataForGame (board, minStep, countStore, StartPos, Stores);
    		return data;
    	}catch (NullPointerException e){
    		throw new ReaderException (" XML  . NullPointerException");
    	}catch (ArrayIndexOutOfBoundsException e){
    		throw new ReaderException (" XML  . ArrayIndexOutOfBoundsException");
    	}catch (NumberFormatException e){
    		throw new ReaderException (" XML  . NumberFormatException");
    	}    	
    }

	/**
	 *   Document  
	 * @return   Score
	 */
    public ArrayList<Score> setScoresFromDoc (){
		try{
	    	ArrayList<Score> arrScores = new ArrayList<Score> (6);
	    	NodeList list = myDoc.getElementsByTagName("item"); 
	    	for(int i=0; i < 5; i++){
	    		Score score = new Score();
	    		score.setName( ((Element)list.item(i)).getAttribute("name") );
	    		score.setScore( ((Element)list.item(i)).getAttribute("score"));
	    		arrScores.add(i, score);
	    	}	    	
	    	return arrScores;  
		}catch (NullPointerException e){
    		throw new ReaderException (" XML  . NullPointerException");
    	}catch (ArrayIndexOutOfBoundsException e){
    		throw new ReaderException (" XML  . ArrayIndexOutOfBoundsException");
    	}catch (NumberFormatException e){
    		throw new ReaderException (" XML  . NumberFormatException");
    	}  
	}
}