package org.amse.mARICa.view;

import java.awt.*;
import java.net.URL;

import javax.swing.*;

import org.amse.mARICa.game.Direction;
import org.amse.mARICa.model.*;

public class BoardPanel extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 *  
	 */
	private IBoard myBoard;
	/**
	 *  
	 */
	private int myHeight;
	/**
	 *  
	 */
	private int myWidth;
	/**
	 *  
	 */
	private int size = 40;
	/**
	 *  
	 */
	private Direction myDirMan;
	/**
	 *    
	 */
	private Color fonColor = this.getBackground();
	
	private Image myIWall;
	private Image myICell;
	private Image myIBox;
	private Image myIManR;
	private Image myIManL;
	private Image myIManU;
	private Image myIManD;
	private Image myIStore;
	private Image myIBoxOnStore;

	public BoardPanel (IBoard board, int height, int width){
		myBoard = board;
		myHeight = height;
		myWidth = width;
		myDirMan = Direction.UP;
		ClassLoader cl = View.class.getClassLoader();
		URL urlIWall = cl.getResource("images/wall1.png");
		URL urlICell= cl.getResource("images/cell15.png");
		URL urlIBox = cl.getResource("images/box.png");
		URL urlIManR = cl.getResource("images/moleR.png");
		URL urlIManL = cl.getResource("images/moleL.png");
		URL urlIManU = cl.getResource("images/moleU.png");
		URL urlIManD = cl.getResource("images/moleD.png");
		URL urlIStore = cl.getResource("images/storeMesh.png");
		URL urlIBoxOnStore = cl.getResource("images/boxOnStore.png");
	    myIWall = new ImageIcon(urlIWall).getImage();
	    myICell = new ImageIcon(urlICell).getImage();
	    myIBox = new ImageIcon(urlIBox).getImage();
	    myIManR = new ImageIcon(urlIManR).getImage();
		myIManL = new ImageIcon(urlIManL).getImage();
		myIManU = new ImageIcon(urlIManU).getImage();
		myIManD = new ImageIcon(urlIManD).getImage();
	    myIStore = new ImageIcon(urlIStore).getImage();
	    myIBoxOnStore = new ImageIcon(urlIBoxOnStore).getImage();   
	}
	/** 
	 *   
	 * @param board
	 * @param height
	 * @param width
	 */
	public void setBoard(IBoard board, int height, int width){
		myBoard = board;
		myHeight = height;
		myWidth = width;
	}
	
	public void setDirMan(Direction dir){
		myDirMan = dir;
	}
	
	/**
	 *    [x][y]
	 * @param x -   
	 * @param y -   
	 */
	private void drawNothing(Graphics g, int x, int y){
		g.setColor(fonColor);
		g.fillRect(x, y, size, size);
	}
	
	/**
	 *   [x][y]  
	 * @param x -   
	 * @param y -   
	 */
	private void drawWall(Graphics g, int x, int y){
		g.drawImage(myIWall, x, y, size, size, fonColor, this);
	}
	
	
	/**
	 *   [x][y]    
	 * @param x -   
	 * @param y -   
	 */
	private void drawBoxOnStore(Graphics g, int x, int y){
		g.drawImage(myICell, x, y, size, size, this);
		g.drawImage(myIBoxOnStore, x, y, size, size, this);
	}
	
	/**
	 *   [x][y]    
	 * @param x -   
	 * @param y -   
	 */
	private void drawManOnStore(Graphics g, int x, int y){
		g.drawImage(myICell, x, y, size, size, this);
		g.drawImage(myIStore, x, y, size, size, this);
		switch (myDirMan){
		case UP:
			g.drawImage(myIManU, x, y, size, size, this);
			break;
		case RIGHT:
			g.drawImage(myIManR, x, y, size, size, this);
			break;
		case DOWN:
			g.drawImage(myIManD, x, y, size, size, this);
			break;
		case LEFT:
			g.drawImage(myIManL, x, y, size, size, this);
			break;
	}	
	}
	
	/**
	 *   [x][y]  
	 * @param x -   
	 * @param y -   
	 */
	private void drawEmptyOnStore(Graphics g, int x, int y){
		g.drawImage(myICell, x, y, size, size, this);
		g.drawImage(myIStore, x, y, size, size, this);
	}
	
	/**
	 *   [x][y]  
	 * @param x -   
	 * @param y -   
	 */
	private void drawBox(Graphics g, int x, int y){
		g.drawImage(myICell, x, y, size, size, this);
		g.drawImage(myIBox, x, y, size, size, this);
	}
	
	/**
	 *   [x][y]  
	 * @param x -   
	 * @param y -   
	 */
	private void drawMan(Graphics g, int x, int y){
		g.drawImage(myICell, x, y, size, size, this);
		switch (myDirMan){
			case UP:
				g.drawImage(myIManU, x, y, size, size, this);
				break;
			case RIGHT:
				g.drawImage(myIManR, x, y, size, size, this);
				break;
			case DOWN:
				g.drawImage(myIManD, x, y, size, size, this);
				break;
			case LEFT:
				g.drawImage(myIManL, x, y, size, size, this);
				break;
		}		
	}
	
	/**
	 *    [x][y]
	 * @param x -   
	 * @param y -   
	 */
	private void drawEmpty(Graphics g, int x, int y){
		g.drawImage(myICell, x, y, size, size, this);
	}
	
	/**
	 *   
	 */
	public void paint(Graphics g) {
		g.clearRect(0, 0, this.getWidth(), this.getHeight());
		g.setColor(fonColor);
		g.fillRect(0, 0, getWidth(), getHeight() );
		for(int i = 0, y = 20; i < myHeight; i++, y += size){
			for(int j = 0, x = 20; j < myWidth; j++, x += size){
				ICell cell = myBoard.getCell(i, j);
				if (cell == null){
					drawNothing(g, x, y);
				} else if (cell instanceof IWall) {
					drawWall(g, x,y);
				} else {
					IFreeCell freeCell = (IFreeCell)cell;
					IEntity entity = freeCell.entity();
					if (freeCell.isStore()) {
						if (entity instanceof IBox) {
							drawBoxOnStore(g, x, y);
						} else if (entity instanceof IMan) {
							drawManOnStore(g, x, y);
						} else /* if (entity == null)*/ {
							drawEmptyOnStore(g,x,y);
						}
					} else {
						if (entity instanceof IBox) {
							drawBox(g,x,y);
						} else if (entity instanceof IMan) {
							drawMan(g,x,y);
						} else /* if (entity == null) */ {
							drawEmpty(g,x,y);
						}
					}			
				}			
			}				
		}
	}
}
